<?php

class TC_PaygateHeleket_ControllerPublic_Heleket extends XenForo_ControllerPublic_Abstract
{
	/**
	 * Enforce registered-users only for all actions in this controller
	 *
	 * @see library/XenForo/XenForo_Controller#_preDispatch($action)
	 */
	protected function _preDispatch($action)
	{
		$this->_assertRegistrationRequired();
	}

	/**
	 * @return \XenForo_ControllerResponse_Error|\XenForo_ControllerResponse_View
	 * @throws \XenForo_Exception
	 */
	public function actionSelectCurrency()
	{
		$paymentFormData = $this->_input->filterSingle('data', XenForo_Input::JSON_ARRAY);
		$signature = $this->_input->filterSingle('sign', XenForo_Input::STRING);

		/** @var TC_PaygateHeleket_Processor_Heleket $handler */
		$handler = $this->getProcessor();
		$xfOptions = XenForo_Application::getOptions();

		$dataString = json_encode($paymentFormData, JSON_UNESCAPED_UNICODE);

		// Check signature
		if (!hash_equals($signature, $handler->getSignature($dataString)))
		{
			return $this->responseError(new XenForo_Phrase('tc_pg_heleket_errors_invalid_data'));
		}

		$viewParams = [
			'data'               => $dataString,
			'sign'               => $handler->getSignature(json_encode($paymentFormData, JSON_UNESCAPED_UNICODE)),
			'currenciesNetworks' => $xfOptions->tcPaygateHeleket_CurrenciesNetworks
		];

		return $this->responseView('', 'tc_pg_heleket_select_currency', $viewParams);
	}

	/**
	 * @return \XenForo_ControllerResponse_Error|\XenForo_ControllerResponse_View
	 * @throws \XenForo_Exception
	 */
	public function actionSelectNetwork()
	{
		$paymentFormData = $this->_input->filterSingle('data', XenForo_Input::JSON_ARRAY);
		$signature = $this->_input->filterSingle('sign', XenForo_Input::STRING);

		$currency = $this->_input->filterSingle('currency', XenForo_Input::STRING);

		/** @var TC_PaygateHeleket_Processor_Heleket $handler */
		$handler = $this->getProcessor();
		$xfOptions = XenForo_Application::getOptions();

		$dataString = json_encode($paymentFormData, JSON_UNESCAPED_UNICODE);

		// Check signature
		if (!hash_equals($signature, $handler->getSignature($dataString)))
		{
			return $this->responseError(new XenForo_Phrase('tc_pg_heleket_errors_invalid_data'));
		}

		$currenciesNetworks = $xfOptions->tcPaygateHeleket_CurrenciesNetworks;
		if (empty($currenciesNetworks[$currency]))
		{
			return $this->responseError(
				new XenForo_Phrase('tc_pg_heleket_errors_invalid_currency_or_network')
			);
		}

		$viewParams = [
			'data'     => $dataString,
			'sign'     => $handler->getSignature(json_encode($paymentFormData, JSON_UNESCAPED_UNICODE)),
			'currency' => $currency,
			'networks' => $currenciesNetworks[$currency]
		];

		return $this->responseView('', 'tc_pg_heleket_select_network', $viewParams);
	}

	/**
	 * @return \XenForo_ControllerResponse_Error|\XenForo_ControllerResponse_Redirect
	 * @throws \XenForo_Exception
	 */
	public function actionInitiate()
	{
		$paymentFormData = $this->_input->filterSingle('data', XenForo_Input::JSON_ARRAY);
		$signature = $this->_input->filterSingle('sign', XenForo_Input::STRING);

		$cryptoCurrency = $this->_input->filterSingle('currency', XenForo_Input::STRING);
		$cryptoNetwork = $this->_input->filterSingle('network', XenForo_Input::STRING);

		/** @var TC_PaygateHeleket_Processor_Heleket $handler */
		$handler = $this->getProcessor();
		$xfOptions = XenForo_Application::getOptions();

		$dataString = json_encode($paymentFormData, JSON_UNESCAPED_UNICODE);

		// Check signature
		if (!hash_equals($signature, $handler->getSignature($dataString)))
		{
			return $this->responseError(new XenForo_Phrase('tc_pg_heleket_errors_invalid_data'));
		}

		if (!empty($cryptoCurrency) && !empty($cryptoNetwork))
		{
			$paymentParams = [
				'amount'          => number_format($paymentFormData['amount'], 2, '.', ''),
				'currency'        => $paymentFormData['currency'],
				'to_currency'     => $cryptoCurrency,
				'network'         => $cryptoNetwork,
				'order_id'        => hash('md5', $paymentFormData['item'] . time()),
				'additional_data' => json_encode([
					'url_return' => $paymentFormData['return_url'],
					'item_id'    => $paymentFormData['item']
				]),

				'url_return'   => $paymentFormData['return_url'],
				'url_callback' => $paymentFormData['callback_url']
			];

			$invoice = $handler->apiCall('v1/payment', $paymentParams, $error);
			if ($error)
			{
				return $this->responseError($error);
			}

			$returnType = isset($xfOptions->tcPaygateHeleket_Type) ? $xfOptions->tcPaygateHeleket_Type : 'internal';

			if ($returnType == 'redirect')
			{
				return $this->responseRedirect(XenForo_ControllerResponse_Redirect::SUCCESS, $invoice['url']);
			}

			return $this->responseRedirect(
				XenForo_ControllerResponse_Redirect::SUCCESS,
				XenForo_Link::buildPublicLink('tc-pg-heleket/status', ['uuid' => $invoice['uuid']])
			);
		}

		return $this->responseError(new XenForo_Phrase('tc_pg_heleket_errors_invalid_currency_or_network'));
	}

	/**
	 * @return \XenForo_ControllerResponse_Error|\XenForo_ControllerResponse_Redirect|\XenForo_ControllerResponse_View
	 * @throws \XenForo_Exception
	 */
	public function actionStatus()
	{
		$invoiceId = $this->_input->filterSingle('uuid', XenForo_Input::STRING);

		/** @var TC_PaygateHeleket_Processor_Heleket $handler */
		$handler = $this->getProcessor();
		$xfOptions = XenForo_Application::getOptions();

		// Payment already created. Fetch info from Heleket
		$payment = $handler->apiCall('v1/payment/info',
			[
				'uuid' => $invoiceId
			],
			$error
		);

		if ($error)
		{
			return $this->responseError(new XenForo_Phrase('tc_pg_heleket_errors_payment_not_found'));
		}

		if (in_array($payment['status'], [
			'paid',
			'paid_over'
		]))
		{
			$additionalData = json_decode($payment['additional_data'], true);

			return $this->responseRedirect(XenForo_ControllerResponse_Redirect::SUCCESS,
				$additionalData['url_return'] ?? XenForo_Application::getOptions()->boardUrl
			);
		}

		if ($payment['expired_at'] <= XenForo_Application::$time)
		{
			return $this->responseError(new XenForo_Phrase('tc_pg_heleket_errors_invoice_expired'));
		}

		$viewParams = [
			'invoice'      => $payment,
			'statusPhrase' => new XenForo_Phrase('tc_pg_heleket_invoice_status_' . $payment['status']),
			'paymentLink'  => $handler->generatePaymentLink($payment['network'], $payment['address'], $payment['payer_amount'])
		];

		return $this->responseView('', 'tc_pg_heleket_invoice_status', $viewParams);
	}

	/**
	 * @return \bdPaygate_Processor_Abstract
	 * @throws \XenForo_Exception
	 */
	protected function getProcessor()
	{
		/** @var TC_PaygateHeleket_Processor_Heleket $processor */
		return bdPaygate_Processor_Abstract::create('TC_PaygateHeleket_Processor_Heleket');
	}
}
